from .profiles import LoggingProfile, ProfileName, make_profile


def init(profile: ProfileName | LoggingProfile = "default") -> None:
    if isinstance(profile, str):
        profile = make_profile(profile)
    profile.init()



import types
from collections.abc import Iterable

import attrs
from rich.traceback import LOCALS_MAX_LENGTH, LOCALS_MAX_STRING, Traceback


@attrs.define
class RichTracebackConfig:
    width: int | None = attrs.field(default=None)
    code_width: int | None = attrs.field(default=None)
    extra_lines: int = attrs.field(default=3)
    theme: str | None = attrs.field(default=None)
    word_wrap: bool = attrs.field(default=False)
    show_locals: bool = attrs.field(default=True)
    locals_max_length: int = attrs.field(default=LOCALS_MAX_LENGTH)
    locals_max_string: int = attrs.field(default=LOCALS_MAX_STRING)
    locals_hide_dunder: bool = attrs.field(default=True)
    locals_hide_sunder: bool = attrs.field(default=False)
    indent_guides: bool = attrs.field(default=True)
    suppress: Iterable[str | types.ModuleType] = attrs.field(default=())
    max_frames: int = attrs.field(default=100)

    def from_exception(
        self,
        exc_type: type[BaseException],
        exc_value: BaseException,
        traceback: types.TracebackType | None,
        /,
    ) -> Traceback:
        return Traceback.from_exception(
            exc_type,
            exc_value,
            traceback,
            width=self.width,
            code_width=self.code_width,
            extra_lines=self.extra_lines,
            theme=self.theme,
            word_wrap=self.word_wrap,
            show_locals=self.show_locals,
            locals_max_length=self.locals_max_length,
            locals_max_string=self.locals_max_string,
            locals_hide_dunder=self.locals_hide_dunder,
            locals_hide_sunder=self.locals_hide_sunder,
            indent_guides=self.indent_guides,
            suppress=self.suppress,
            max_frames=self.max_frames,
        )



from . import mixins
from ._abc import LoggingProfile
from ._cherries import LoggingProfileCherries
from ._default import LoggingProfileDefault
from ._factory import ProfileName, make_profile
from .mixins import (
    LoggingProfileMixinExceptHook,
    LoggingProfileMixinLoguru,
    LoggingProfileMixinUnraisableHook,
    ic_arg_to_string_function,
)

__all__ = [
    "LoggingProfile",
    "LoggingProfileCherries",
    "LoggingProfileDefault",
    "LoggingProfileMixinExceptHook",
    "LoggingProfileMixinLoguru",
    "LoggingProfileMixinUnraisableHook",
    "ProfileName",
    "ic_arg_to_string_function",
    "make_profile",
    "mixins",
]



import functools
from collections.abc import Callable, Mapping
from typing import Any, Protocol, overload

import wrapt


class Decorator(Protocol):
    def __call__[T](self, wrapped: T, /) -> T: ...


class Wrapper(Protocol):
    def __call__(
        self, wrapped: Any, instance: Any, args: tuple, kwargs: dict[str, Any], /
    ) -> Any: ...


def function_wrapper(
    attrs: Mapping[str, Any] | None = None,
) -> type[wrapt.FunctionWrapper]:
    if not attrs:
        return wrapt.FunctionWrapper

    class BoundFunctionWrapper(wrapt.BoundFunctionWrapper):
        def __init__(self, *args, **kwargs) -> None:
            super().__init__(*args, **kwargs)
            for key, value in attrs.items():
                setattr(self, key, value)

    class FunctionWrapper(wrapt.FunctionWrapper):
        __bound_function_wrapper__ = BoundFunctionWrapper

        def __init__(self, *args, **kwargs) -> None:
            super().__init__(*args, **kwargs)
            for key, value in attrs.items():
                setattr(self, key, value)

    return FunctionWrapper


@overload
def decorator(
    wrapper: Wrapper,
    enabled: bool | Callable[[], None] | None = None,
    adapter: Any = None,
    proxy: type | None = None,
    *,
    attrs: Mapping[str, Any] | None = None,
) -> Decorator: ...
@overload
def decorator(
    wrapper: None = None,
    enabled: bool | Callable[[], None] | None = None,
    adapter: Any = None,
    proxy: type | None = None,
    *,
    attrs: Mapping[str, Any] | None = None,
) -> Callable[[Wrapper], Decorator]: ...
def decorator[T](
    wrapper: Callable | None = None,
    enabled: bool | Callable[[], None] | None = None,
    adapter: Any = None,
    proxy: type[T] | None = None,
    *,
    attrs: Mapping[str, Any] | None = None,
) -> Any:
    if wrapper is None:
        return functools.partial(
            decorator,
            enabled=enabled,
            adapter=adapter,
            proxy=proxy,
            attrs=attrs,
        )
    if proxy is None:
        proxy = function_wrapper(attrs)
    return wrapt.decorator(wrapper, enabled=enabled, adapter=adapter, proxy=proxy)



from typing import Literal

from ._abc import LoggingProfile

# make sure profiles are registered
from ._cherries import LoggingProfileCherries  # noqa: F401
from ._default import LoggingProfileDefault  # noqa: F401

# for code-completion
type ProfileName = Literal["default", "cherries"] | str  # noqa: PYI051


def make_profile(name: ProfileName = "default", /, *args, **kwargs) -> LoggingProfile:
    return LoggingProfile[name](*args, **kwargs)



from ._excepthook import LoggingProfileMixinExceptHook
from ._icecream import LoggingProfileMixinIcecream, ic_arg_to_string_function
from ._loguru import LoggingProfileMixinLoguru
from ._stdlib import LoggingProfileMixinStdlib
from ._unraisablehook import LoggingProfileMixinUnraisableHook

__all__ = [
    "LoggingProfileMixinExceptHook",
    "LoggingProfileMixinIcecream",
    "LoggingProfileMixinLoguru",
    "LoggingProfileMixinStdlib",
    "LoggingProfileMixinUnraisableHook",
    "ic_arg_to_string_function",
]
